package com.b3dgs.tyrian;

import com.b3dgs.lionengine.drawable.TiledSprite;
import com.b3dgs.lionengine.game.Camera;
import com.b3dgs.lionengine.utility.Maths;
import com.b3dgs.lionengine.utility.Timer;
import java.awt.Color;
import java.awt.Graphics2D;

public class Background {

    private class Star {

        private final float vx, vy;
        private final int id;
        private float x, y;

        private Star(float x, float y, float vx, float vy, int id) {
            this.x = x;
            this.y = y;
            this.vx = vx;
            this.vy = vy;
            this.id = id;
        }

        private void update(float extrp) {
            this.x += this.vx * extrp;
            this.y += this.vy * extrp;
            if (this.y > 210) {
                this.y = -10;
                this.x = Maths.random(-20, 340);
            }
        }
    }
    private final int normal = 200;
    private final TiledSprite surface;
    private final Background.Star[] stars;
    private final Color[] colors;
    private final Timer effectTime;
    private float ic;
    private int c, to;
    private boolean effect;

    public Background(RessourcesHandler rh) {
        this.colors = new Color[this.normal + 1];
        for (int i = 0; i < this.colors.length; i++) {
            this.colors[i] = new Color(0, 0, 0, i);
        }
        this.effectTime = new Timer();
        this.c = this.normal;
        this.to = this.c;
        this.ic = this.to;
        this.surface = rh.getTiledSprite(ListRessources.Stars);
        this.stars = new Background.Star[256];
        for (int i = 0; i < this.stars.length; i++) {
            int id = Maths.random(0, 5);
            this.stars[i] = new Background.Star(Maths.random(-20, 340), Maths.random(-10, 200), 0.0f, (Maths.random(20, 50) / 15.f) * 1.0f, id);
        }
        this.effectTime.start();
    }

    public void update(float extrp) {
        this.ic = Maths.curveValue(this.ic, this.to, 120.0f);
        this.c = (int) Math.floor(this.ic);
        for (int i = 0; i < this.stars.length; i++) {
            this.stars[i].update(extrp);
        }

        if (Maths.random(0, 1000) == 0 && this.effectTime.elapsed(5000)) {
            this.effect();
            this.effectTime.start();
        }
    }

    public void render(Graphics2D g, Camera camera) {
        g.setColor(this.colors[this.c]);
        g.fillRect(0, 0, 320, 200);
        for (int i = 0; i < this.stars.length; i++) {
            this.surface.render(g, this.stars[i].id, (int) this.stars[i].x - camera.getX(), (int) this.stars[i].y);
        }
        if (this.effect) {
            g.setColor(Color.WHITE);
            g.fillRect(0, 0, 320, 200);
            this.effect = false;
        }
    }

    public void effect() {
        this.ic = 25;
        this.to = this.normal;
        this.effect = true;
    }

    public void setAlpha(int c) {
        this.c = c;
    }
}
