package com.b3dgs.tyrian;

import com.b3dgs.lionengine.utility.Maths;
import com.b3dgs.tyrian.bonus.BonusCoin;
import com.b3dgs.tyrian.bonus.BonusWeaponFront;
import com.b3dgs.tyrian.bonus.BonusWeaponRear;
import com.b3dgs.tyrian.bonus.ListCoin;
import com.b3dgs.tyrian.bonus.PowerUp;
import com.b3dgs.tyrian.gameplay.ListWeaponFront;
import com.b3dgs.tyrian.gameplay.ListWeaponRear;

public class BonusFactory {

    private final ListWeaponFront[] weaponsFront = ListWeaponFront.values();
    private final ListWeaponRear[] weaponsRear = ListWeaponRear.values();
    private final ListCoin[] coins = ListCoin.values();
    private final RessourcesHandler rh;
    private final EntitiesHandler handler;

    public BonusFactory(RessourcesHandler rh, EntitiesHandler handler) {
        this.rh = rh;
        this.handler = handler;
    }

    public ListWeaponFront getRndWeaponFront() {
        return this.weaponsFront[Maths.random(0, this.weaponsFront.length - 1)];
    }

    public ListWeaponRear getRndWeaponRear() {
        return this.weaponsRear[Maths.random(0, this.weaponsRear.length - 1)];
    }

    public BonusWeaponFront createBonusWeaponFront(ListWeaponFront weapon) {
        switch (weapon) {
            case HeavyMissilesFront:
                return new BonusWeaponFront(this.rh, ListWeaponFront.HeavyMissilesFront, 9);
            case HyperPulse:
                return new BonusWeaponFront(this.rh, ListWeaponFront.HyperPulse, 1);
            case Laser:
                return new BonusWeaponFront(this.rh, ListWeaponFront.Laser, 3);
            case MissileLauncher:
                return new BonusWeaponFront(this.rh, ListWeaponFront.MissileLauncher, 8);
            case MultiCannon:
                return new BonusWeaponFront(this.rh, ListWeaponFront.MultiCannon, 2);
            case Proton:
                return new BonusWeaponFront(this.rh, ListWeaponFront.Proton, 5);
            case PulseCannon:
                return new BonusWeaponFront(this.rh, ListWeaponFront.PulseCannon, 0);
            case SonicWaveFront:
                return new BonusWeaponFront(this.rh, ListWeaponFront.SonicWaveFront, 12);
            case VulcanCannonFront:
                return new BonusWeaponFront(this.rh, ListWeaponFront.VulcanCannonFront, 6);
            case ZicaLaser:
                return new BonusWeaponFront(this.rh, ListWeaponFront.ZicaLaser, 4);
            default:
                return new BonusWeaponFront(this.rh, ListWeaponFront.PulseCannon, 0);
        }
    }

    public BonusWeaponRear createBonusWeaponRear(ListWeaponRear weapon) {
        switch (weapon) {
            case HeavyMissilesRear:
                return new BonusWeaponRear(this.rh, ListWeaponRear.HeavyMissilesRear, 24);
            case RearCannon:
                return new BonusWeaponRear(this.rh, ListWeaponRear.RearCannon, 18);
            case SonicWaveRear:
                return new BonusWeaponRear(this.rh, ListWeaponRear.SonicWaveRear, 19);
            case VulcanCannonRear:
                return new BonusWeaponRear(this.rh, ListWeaponRear.VulcanCannonRear, 20);
            default:
                return new BonusWeaponRear(this.rh, ListWeaponRear.SonicWaveRear, 19);
        }
    }

    public ListCoin getRndCoin() {
        return this.coins[Maths.random(0, this.coins.length - 1)];
    }

    public BonusCoin createBonusCoin(ListCoin coin) {
        switch (coin) {
            case Green:
                return new BonusCoin(this.rh, this.handler, 0, 10, 0, 3);
            case Grey:
                return new BonusCoin(this.rh, this.handler, 11, 25, 3, 3);
            case Brown:
                return new BonusCoin(this.rh, this.handler, 22, 50, 6, 3);
            case Orange:
                return new BonusCoin(this.rh, this.handler, 33, 75, 9, 3);
            default:
                return null;
        }
    }

    public PowerUp createPowerUp() {
        return new PowerUp(this.rh);
    }
}
