package com.b3dgs.tyrian;

import com.b3dgs.lionengine.file.FileReader;
import com.b3dgs.lionengine.game.map.AbstractTileMap;
import com.b3dgs.lionengine.game.map.DefaultTile;
import com.b3dgs.lionengine.game.shmup.EffectHandler;
import com.b3dgs.tyrian.entity.layer1.AbstractLayer1Entity;
import com.b3dgs.tyrian.entity.layer1.level1.BlackSpider;
import com.b3dgs.tyrian.entity.layer1.level1.BrownGenerator;
import com.b3dgs.tyrian.entity.layer1.level1.BrownLamp;
import com.b3dgs.tyrian.entity.layer1.level1.DoubleCannon;
import com.b3dgs.tyrian.entity.layer1.level1.DoublePulse;
import com.b3dgs.tyrian.entity.layer1.level1.GreenPulse;
import com.b3dgs.tyrian.entity.layer1.level1.House812;
import com.b3dgs.tyrian.entity.layer1.level1.House92;
import com.b3dgs.tyrian.entity.layer1.level1.OpenablePulse;
import com.b3dgs.tyrian.entity.layer1.level1.Pump;
import com.b3dgs.tyrian.entity.layer1.level1.RedBox;
import com.b3dgs.tyrian.entity.layer1.level1.RedEngine;
import com.b3dgs.tyrian.entity.layer1.level1.Silo;
import com.b3dgs.tyrian.entity.layer1.level1.SingleCannon;
import com.b3dgs.tyrian.entity.layer1.level1.SubSquare;
import com.b3dgs.tyrian.entity.layer1.level1.ThreeHRed;
import com.b3dgs.tyrian.entity.layer1.level1.ThreeVRed;
import java.io.IOException;

public class Map extends AbstractTileMap<DefaultTile> {

    private final RessourcesHandler rh;
    private final Entity1Handler entityLayer1;
    private final EntitiesHandler handler;
    private final EffectHandler effects;
    private int offsetY;

    public Map(RessourcesHandler rh, Entity1Handler entityLayer1, EntitiesHandler handler, EffectHandler effects) {
        super("tiles", 24, 28, 200);
        this.rh = rh;
        this.entityLayer1 = entityLayer1;
        this.handler = handler;
        this.effects = effects;
    }

    public void setOffsetY(float y) {
        this.offsetY = (int) Math.floor(y / 28.0);
    }

    @Override
    public void load(FileReader fr) throws IOException {
        super.load(fr);
        for (int v = 0; v < this.heightInTile; v++) {
            for (int h = 0; h < this.widthInTile; h++) {
                DefaultTile tile = this.getTile(v, h);
                if (tile != null) {
                    this.add(this.get(tile.getNumber(), h, v + this.offsetY));
                }
            }
        }
    }

    private AbstractLayer1Entity get(int id, int x, int y) {
        switch (id) {
            case 6:
                return new SingleCannon(this.rh, this.effects, x, y);
            case 7:
                return new DoubleCannon(this.rh, this.handler, this.effects, x, y);
            case 8:
                return new RedEngine(this.rh, this.effects, x, y);
            case 9:
                return new GreenPulse(this.rh, this.handler, this.effects, x, y);
            case 19:
                return new OpenablePulse(this.rh, this.effects, x, y);
            case 23:
                return new BrownGenerator(this.rh, this.effects, x, y);
            case 26:
                return new DoublePulse(this.rh, this.handler, this.effects, x, y);
            case 28:
                return new BrownLamp(this.rh, this.effects, x, y);
            case 30:
                return new BlackSpider(this.rh, this.effects, x, y);
            case 32:
                return new SubSquare(this.rh, this.effects, x, y);
            case 45:
                return new Silo(this.rh, this.effects, x, y);
            case 46:
                return new RedBox(this.rh, this.effects, x, y);
            case 56:
                return new ThreeVRed(this.rh, this.effects, x, y);
            case 60:
                return new Pump(this.rh, this.effects, x, y);
            case 62:
                return new House92(this.rh, this.effects, x, y);
            case 64:
                return new House812(this.rh, this.effects, x, y);
            case 66:
                return new ThreeHRed(this.rh, this.effects, x, y);
            default:
                return null;
        }
    }

    private void add(AbstractLayer1Entity entity) {
        if (entity != null) {
            this.entityLayer1.add(entity);
        }
    }

    @Override
    public DefaultTile createTile(int pattern, int number, int x, int y, String collision) {
        return new DefaultTile(pattern, number, x, y, collision);
    }
}
