package com.b3dgs.tyrian;

import com.b3dgs.lionengine.Audio;
import com.b3dgs.lionengine.Drawable;
import com.b3dgs.lionengine.audio.Midi;
import com.b3dgs.lionengine.core.Alignment;
import com.b3dgs.lionengine.drawable.FontSprite;
import com.b3dgs.lionengine.drawable.Sprite;
import com.b3dgs.lionengine.drawable.TiledSprite;
import com.b3dgs.lionengine.engine.Loader;
import com.b3dgs.lionengine.input.Keyboard;
import com.b3dgs.lionengine.input.Mouse;
import com.b3dgs.lionengine.utility.Media;
import com.b3dgs.tyrian.gameplay.AbstractMenuButton;
import java.awt.Color;
import java.awt.Graphics2D;

public class Menu extends AbstractSequence {

    private final Color colors[];
    private final TiledSprite buttonsSurfaces;
    private final Sprite surfaces[], cursor, armory, ship;
    private final FontSprite font, font2;
    private final Midi midi;
    private final AbstractMenuButton[] buttons;
    private int alpha, alphaRemake, menu, titleY;
    private int mx, my;
    private boolean used;

    public Menu(Loader loader) {
        super(loader);
        this.colors = new Color[256];
        for (int i = 0; i < 256; i++) {
            this.colors[i] = new Color(255, 255, 255, i);
        }
        this.buttonsSurfaces = Drawable.DRAWABLE.loadTiledSprite(Media.get("menu", "buttons.png"), 96, 16);
        this.buttonsSurfaces.load(true);
        this.buttonsSurfaces.setAlpha(0);
        this.font = Drawable.DRAWABLE.createFontSprite(Media.get("misc", "font.png"), Media.get("misc", "fontdata.xml"), 8, 9);
        this.font.load(true);
        this.font.setAlpha(0);
        this.font2 = Drawable.DRAWABLE.createFontSprite(Media.get("misc", "font2.png"), Media.get("misc", "fontdata2.xml"), 6, 6);
        this.font2.load(true);
        this.font2.setAlpha(0);
        this.surfaces = new Sprite[5];
        this.cursor = Drawable.DRAWABLE.loadSprite(Media.get("misc", "cursor.png"));
        this.cursor.load(false);
        this.armory = Drawable.DRAWABLE.loadSprite(Media.get("menu", "armory.png"));
        this.armory.load(true);
        this.ship = Drawable.DRAWABLE.loadSprite(Media.get("menu", "ship.png"));
        this.ship.load(true);
        this.midi = Audio.AUDIO.loadMidi(Media.get("musics", "menu.mid"));
        this.buttons = new AbstractMenuButton[3];
    }

    @Override
    protected void load() {
        this.surfaces[0] = Drawable.DRAWABLE.loadSprite(Media.get("menu", "background.png"));
        this.surfaces[1] = Drawable.DRAWABLE.loadSprite(Media.get("menu", "tyrian.png"));
        this.surfaces[2] = Drawable.DRAWABLE.loadSprite(Media.get("menu", "remake.png"));
        this.surfaces[3] = Drawable.DRAWABLE.loadSprite(Media.get("menu", "start.png"));
        this.surfaces[4] = Drawable.DRAWABLE.loadSprite(Media.get("menu", "howto.png"));

        for (int i = 0; i < this.surfaces.length; i++) {
            this.surfaces[i].load(true);
        }

        this.buttons[0] = new AbstractMenuButton(this.buttonsSurfaces, 0, 112, 130) {

            @Override
            public void onClick() {
                super.onClick();
                alpha = 255;
                menu = 4;
            }
        };
        this.buttons[1] = new AbstractMenuButton(this.buttonsSurfaces, 2, 112, 145) {

            @Override
            public void onClick() {
                super.onClick();
                alpha = 255;
                menu = 5;
            }
        };
        this.buttons[2] = new AbstractMenuButton(this.buttonsSurfaces, 4, 112, 160) {

            @Override
            public void onClick() {
                super.onClick();
                alpha = 255;
                menu = 3;
            }
        };

        this.alpha = 0;
        this.menu = 0;
        this.titleY = 72;
        this.screen.hideCursor();
        this.midi.play(true);
        this.used = true;
    }

    @Override
    protected void update(float extrp) {
        super.update(extrp);
        this.mx = this.mouse.getOnWindowX();
        this.my = this.mouse.getOnWindowY();
        if (this.keyboard.isPressed(Keyboard.ESCAPE)) {
            this.end();
        }
        switch (this.menu) {
            case 0: // Incomming
                this.alpha += 5;
                if (this.alpha > 255) {
                    this.alpha = 255;
                }
                this.titleY -= 1;
                if (this.titleY < 0) {
                    this.titleY = 0;
                }
                this.alphaRemake += 3;
                if (this.alphaRemake > 200) {
                    this.alphaRemake = 200;
                }
                if (this.alphaRemake == 200 && this.titleY == 0) {
                    this.menu = 1;
                }
                this.surfaces[0].setAlpha(this.alpha);
                this.surfaces[1].setAlpha(this.alpha);
                this.surfaces[2].setAlpha(this.alphaRemake);
                break;
            case 1: // Buttons incomming
                this.alpha -= 6;
                if (this.alpha < 0) {
                    this.alpha = 0;
                    this.menu = 2;
                }
                this.buttonsSurfaces.setAlpha(255 - this.alpha);
                this.font2.setAlpha(255 - this.alpha);
                break;
            case 2: // Main buttons handling
                for (int i = 0; i < this.buttons.length; i++) {
                    this.buttons[i].update(this.mouse, Mouse.LEFT, extrp);
                }
                break;
            case 3: // Exit
                this.alpha -= 8;
                if (this.alpha < 0) {
                    this.alpha = 0;
                    this.end();
                }
                this.surfaces[0].setAlpha(this.alpha);
                this.surfaces[1].setAlpha(this.alpha);
                this.surfaces[2].setAlpha(this.alpha);
                this.buttonsSurfaces.setAlpha(this.alpha);
                this.font2.setAlpha(this.alpha);
                break;
            case 4: // Start
                this.alpha -= 10;
                if (this.alpha < 0) {
                    this.alpha = 0;
                    this.end(new Scene(this.loader));
                }
                break;
            case 5: // Fade out menu -> how to play
                this.alpha -= 15;
                if (this.alpha < 0) {
                    this.alpha = 0;
                    this.menu = 6;
                    this.surfaces[4].setAlpha(0);
                }
                this.surfaces[0].setAlpha(this.alpha);
                this.surfaces[1].setAlpha(this.alpha);
                this.surfaces[2].setAlpha(this.alpha);
                this.buttonsSurfaces.setAlpha(this.alpha);
                this.font2.setAlpha(this.alpha);
                this.armory.setAlpha(0);
                this.ship.setAlpha(0);
                break;
            case 6: // Fade in How to play
                this.alpha += 15;
                if (this.alpha > 255) {
                    this.alpha = 255;
                    this.used = true;
                    this.menu = 7;
                }
                this.surfaces[4].setAlpha(this.alpha);
                this.font.setAlpha(this.alpha);
                this.armory.setAlpha(this.alpha);
                this.ship.setAlpha(this.alpha);
                break;
            case 7: // How to play
                if (!this.used && (this.keyboard.used() || this.mouse.getMouseClick() > 0)) {
                    this.menu = 8;
                    this.used = true;
                }
                break;
            case 8: // How to play fade out -> menu
                this.alpha -= 15;
                if (this.alpha < 0) {
                    this.alpha = 0;
                    this.menu = 0;
                    this.titleY = 72;
                    this.alphaRemake = 0;
                }
                this.surfaces[4].setAlpha(this.alpha);
                this.font.setAlpha(this.alpha);
                this.armory.setAlpha(this.alpha);
                this.ship.setAlpha(this.alpha);
                break;
        }
        if (this.mouse.getMouseClick() == 0 && !this.keyboard.used()) {
            this.used = false;
        }
    }

    @Override
    protected void render(Graphics2D g) {
        g.clearRect(0, 0, this.width, this.height);
        if (this.menu < 7) {
            this.surfaces[0].render(g, 0, 0);
            this.surfaces[1].render(g, 0, this.titleY);
            this.surfaces[2].render(g, 150, 90 - this.titleY / 4);
            this.font2.draw(g, 2, 180, Alignment.LEFT, "TYRIAN REMAKE", "BYRON 3D GAMES STUDIO", "V" + Main.VERSION);
        }
        if (this.menu >= 6) {
            this.surfaces[4].render(g, 0, 0);
            this.font.draw(g, 160, 2, Alignment.CENTERED, "How to Play ?");
            this.font.draw(g, 4, 24, Alignment.LEFT,
                    "Goal: You have to protect the earth",
                    "by destroying all meteors before",
                    "they reach the bottom of the screen.");

            this.font.draw(g, 4, 60, Alignment.LEFT,
                    "Meteor are comming during a wave.",
                    "Each wave duration is about one minute.",
                    "At the end, you can buy/upgrade weapons",
                    "by putting your ship under the big one.");

            this.font.draw(g, 125, 120, Alignment.LEFT,
                    "-Mouse/arrowkeys to move",
                    "-Click/space to shoot",
                    "-Escape to exit game");
            this.font.draw(g, 265, 170, Alignment.CENTERED, "Press any key", "or", "Do any click");

            this.armory.render(g, 24, 98);
            this.ship.render(g, 140, 170);
            g.setColor(Color.GREEN);
            g.drawLine(70, 162, 140, 180);
        }

        if (this.menu > 0 && this.menu < 7) {
            for (int i = 0; i < this.buttons.length; i++) {
                this.buttons[i].render(g);
            }
        }
        if (this.menu > 1) {
            this.cursor.render(g, this.mx, this.my);
        }
        if (this.menu == 4) {
            g.setColor(this.colors[255 - this.alpha]);
            g.fillRect(0, 0, this.width, this.height);
        }
        super.render(g);
    }

    @Override
    protected void terminate() {
        this.midi.stop();
    }
}
