package com.b3dgs.tyrian;

import com.b3dgs.lionengine.game.shmup.AbstractRessourcesHandler;
import com.b3dgs.lionengine.game.shmup.ShipBulletHandler;
import com.b3dgs.lionengine.utility.Maths;
import com.b3dgs.lionengine.utility.Media;
import com.b3dgs.tyrian.gameplay.ListWeaponFront;
import com.b3dgs.tyrian.gameplay.ListWeaponRear;
import com.b3dgs.tyrian.weapon.Weapon;
import java.util.EnumMap;

public class RessourcesHandler extends AbstractRessourcesHandler<ListRessources> {

    public final ShipBulletHandler phFront, phRear;
    public final BulletFactory bulletFactory;
    private final EnumMap<ListWeaponFront, Weapon> weaponsFront = new EnumMap<ListWeaponFront, Weapon>(ListWeaponFront.class);
    private final EnumMap<ListWeaponRear, Weapon> weaponsRear = new EnumMap<ListWeaponRear, Weapon>(ListWeaponRear.class);
    private static final ListWeaponFront[] fronts = ListWeaponFront.values();
    private static final ListWeaponRear[] rears = ListWeaponRear.values();
    private static final String EFFECTS_DIR = "effects";
    private static final String PLAYER_DIR = "player";
    private static final String BACKGROUND_DIR = "background";
    private static final String MISC_DIR = "misc";

    public RessourcesHandler(World world, String lvl) {
        super(ListRessources.class);

        this.addTiledSprite(ListRessources.EntityLayer1, Media.get(lvl, "entity_layer1.png"), 24, 28, false);
        this.addTiledSprite(ListRessources.EntityLayer2, Media.get(lvl, "entity_layer2.png"), 24, 28, false);

        this.addTiledSprite(ListRessources.Bonus1, Media.get(MISC_DIR, "bonus1.png"), 12, 14, false);
        this.addTiledSprite(ListRessources.Bonus2, Media.get(MISC_DIR, "bonus2.png"), 24, 28, false);
        this.addTiledSprite(ListRessources.Bonus3, Media.get(MISC_DIR, "bonus3.png"), 24, 28, false);
        this.addTiledSprite(ListRessources.Weapons, Media.get(MISC_DIR, "weapons.png"), 12, 14, false);
        this.addTiledSprite(ListRessources.Armory, Media.get(MISC_DIR, "armory.png"), 192, 224, false);
        this.addSprite(ListRessources.Cursor, Media.get(MISC_DIR, "cursor.png"), false);
        this.addSprite(ListRessources.Gameover, Media.get(MISC_DIR, "gameover.png"), true);
        this.addSprite(ListRessources.Paused, Media.get(MISC_DIR, "paused.png"), false);

        this.addTiledSprite(ListRessources.Explode1, Media.get(EFFECTS_DIR, "explode1.png"), 24, 28, false);
        this.addTiledSprite(ListRessources.Explode2, Media.get(EFFECTS_DIR, "explode2.png"), 24, 28, false);
        this.addTiledSprite(ListRessources.Explode3, Media.get(EFFECTS_DIR, "explode3.png"), 12, 14, false);
        this.addTiledSprite(ListRessources.BulletHit, Media.get(EFFECTS_DIR, "bullet_hit.png"), 12, 14, false);
        this.addTiledSprite(ListRessources.Smoke, Media.get(EFFECTS_DIR, "smoke.png"), 12, 14, false);
        this.addTiledSprite(ListRessources.Scores, Media.get(EFFECTS_DIR, "scores.png"), 12, 14, false);
        this.addSprite(ListRessources.Hit, Media.get(EFFECTS_DIR, "hit.png"), false);

        this.addTiledSprite(ListRessources.Ships, Media.get(PLAYER_DIR, "ships.png"), 24, 28, false);
        this.addTiledSprite(ListRessources.Equipment, Media.get(PLAYER_DIR, "equipment.png"), 24, 28, false);
        this.addTiledSprite(ListRessources.ShipUpgrade, Media.get(PLAYER_DIR, "ship_upgrade.png"), 96, 112, false);
        this.addSprite(ListRessources.Hud, Media.get(PLAYER_DIR, "hud.png"), false);
        this.addSprite(ListRessources.Buy, Media.get(PLAYER_DIR, "buy.png"), false);
        this.addSprite(ListRessources.Transition, Media.get(PLAYER_DIR, "transition.png"), true);

        this.addTiledSprite(ListRessources.Stars, Media.get(BACKGROUND_DIR, "stars.png"), 3, 3, false);

        this.phFront = new ShipBulletHandler(-12, -world.height - 14, world.width + 12, 14, world.entityLayer1, world.entityLayer2);
        this.phRear = new ShipBulletHandler(-12, -world.height - 14, world.width + 12, 14, world.entityLayer1, world.entityLayer2);
        this.bulletFactory = new BulletFactory(this, world.effects);

        for (ListWeaponFront w : ListWeaponFront.values()) {
            this.instanceFront(w);
        }
        for (ListWeaponRear w : ListWeaponRear.values()) {
            this.instanceRear(w);
        }
    }

    private void instanceFront(ListWeaponFront weapon) {
        this.weaponsFront.put(weapon, this.newWeaponFront(weapon));
    }

    private void instanceRear(ListWeaponRear weapon) {
        this.weaponsRear.put(weapon, this.newWeaponRear(weapon));
    }

    public Weapon newWeaponFront(ListWeaponFront weapon) {
        try {
            Class<?> clazz = Class.forName("com.b3dgs.tyrian.weapon.front." + weapon.name());
            return (Weapon) clazz.getConstructor(RessourcesHandler.class).newInstance(this);
        } catch (Exception ex) {
            return null;
        }
    }

    public Weapon newWeaponRear(ListWeaponRear weapon) {
        try {
            Class<?> clazz = Class.forName("com.b3dgs.tyrian.weapon.rear." + weapon.name());
            return (Weapon) clazz.getConstructor(RessourcesHandler.class).newInstance(this);
        } catch (Exception ex) {
            return null;
        }
    }

    public Weapon getWeaponFront(ListWeaponFront weapon) {
        return this.weaponsFront.get(weapon);
    }

    public Weapon getWeaponRear(ListWeaponRear weapon) {
        return this.weaponsRear.get(weapon);
    }

    public Weapon getRandomFrontWeapon() {
        ListWeaponFront id;
        do {
            id = fronts[Maths.random(0, fronts.length - 1)];
        } while (id == ListWeaponFront.NoneFront);
        return this.newWeaponFront(id);
    }

    public Weapon getRandomRearWeapon() {
        ListWeaponRear id;
        do {
            id = rears[Maths.random(0, rears.length - 1)];
        } while (id == ListWeaponRear.NoneRear);
        return this.newWeaponRear(rears[Maths.random(0, rears.length - 1)]);
    }
}
