package com.b3dgs.tyrian;

import com.b3dgs.lionengine.Audio;
import com.b3dgs.lionengine.audio.Midi;
import com.b3dgs.lionengine.audio.Sound;
import com.b3dgs.lionengine.game.Alterable;
import com.b3dgs.lionengine.utility.Media;
import com.b3dgs.lionengine.utility.Timer;
import java.util.EnumMap;

public class SoundManager {

    public static enum SFX {

        EXPLODE_LITTLE, EXPLODE_MEDIUM, EXPLODE_LARGE,
        WEAPON_PULSE, WEAPON_VULCAN, WEAPON_WAVE, WEAPON_PROTON, WEAPON_MISSILE, WEAPON_LASER, WEAPON_ZICALASER,
        PLAYER_SHIELD, PLAYER_LIFE,
        OBJECT_HIT, BULLET_HIT,
        ITEM, POWERUP, CLICK, ACCEPT, WARNING, MAX, SELECT,
        BLANK
    }
    public static final String DIR = "sfx";
    public static final Alterable VOLUME = new Alterable(100);
    private static final EnumMap<SFX, Sound> SOUNDS = new EnumMap<SFX, Sound>(SFX.class);
    private static final Timer EXPLODE_TIMER = new Timer();

    private SoundManager() {
    }

    public static void init() {
        VOLUME.fill();
        SFX[] sfxs = SFX.values();
        for (int i = 0; i < sfxs.length; i++) {
            loadSound(sfxs[i]);
        }
        play(SFX.BLANK);
        play(SFX.BLANK);
        EXPLODE_TIMER.start();
    }

    private static void loadSound(SFX sfx) {
        SOUNDS.put(sfx, Audio.AUDIO.loadSound(Media.get(DIR, sfx.name().toLowerCase() + ".wav")));
    }

    public static void play(SFX sfx) {
        if (sfx == SFX.EXPLODE_LARGE && !EXPLODE_TIMER.elapsed(300)) {
            return;
        }
        Sound snd = SOUNDS.get(sfx);
        snd.setVolume(VOLUME.getPercent());
        /* int chan = getChannel(sfx);
         * if (chan == -1) {
         * //snd.play();
         * } else {
         * //snd.play(chan);
         * } */
        snd.play();
        if (sfx == SFX.EXPLODE_LARGE) {
            EXPLODE_TIMER.start();
        }
    }

    private static int getChannel(SFX sfx) {
        switch (sfx) {
            case EXPLODE_LITTLE:
                return -1;
            case EXPLODE_MEDIUM:
                return -1;
            case EXPLODE_LARGE:
                return -1;
            case WEAPON_PULSE:
                return 1;
            case WEAPON_VULCAN:
                return 1;
            case WEAPON_WAVE:
                return 1;
            case PLAYER_SHIELD:
                return 2;
            case PLAYER_LIFE:
                return 2;
            case OBJECT_HIT:
                return 3;
            case BULLET_HIT:
                return 4;
            case ITEM:
                return 5;
            case POWERUP:
                return 5;
            default:
                return 0;
        }
    }
}
