package com.b3dgs.tyrian;

import com.b3dgs.lionengine.core.Screen;
import com.b3dgs.lionengine.file.FileReader;
import com.b3dgs.lionengine.file.FileWriter;
import com.b3dgs.lionengine.file.XMLNode;
import com.b3dgs.lionengine.game.shmup.AbstractShmupWorld;
import com.b3dgs.lionengine.game.shmup.EffectHandler;
import com.b3dgs.lionengine.input.Keyboard;
import com.b3dgs.lionengine.input.Mouse;
import com.b3dgs.tyrian.gameplay.InBuy;
import com.b3dgs.tyrian.gameplay.InGame;
import com.b3dgs.tyrian.gameplay.Ship;
import com.b3dgs.tyrian.ship.ListShip;
import com.b3dgs.tyrian.ship.ShipFactory;
import java.awt.Graphics2D;
import java.io.IOException;

public class World extends AbstractShmupWorld {

    public final Entity1Handler entityLayer1;
    public final Entity2Handler entityLayer2;
    public final EntitiesHandler entitiesLayer;
    public final EffectHandler effects;
    public final InGame inGame;
    private InBuy inBuy;
    private RessourcesHandler rh;
    private ShipFactory shipFactory;
    private Ship ship;

    public World(Screen screen) {
        super(screen);
        this.entityLayer1 = new Entity1Handler(-12, 0, this.width + 12, 288 * 3);
        this.entityLayer2 = new Entity2Handler(-12, -28 * 2, this.width + 12, 200);
        this.entitiesLayer = new EntitiesHandler(-12, -28 * 6, this.width + 12, 200);
        this.effects = new EffectHandler();
        this.inGame = new InGame(this.entityLayer1, this.entityLayer2, this.entitiesLayer, this.effects);
    }

    @Override
    public void update(Keyboard keyboard, Mouse mouse, float extrp) {
        if (this.inGame.isBuying()) {
            this.inBuy.update(keyboard, mouse, extrp);
        } else {
            this.inGame.update(keyboard, mouse, extrp);
            if (this.inGame.isBuying()) {
                this.inBuy.start();
            }
        }
    }

    @Override
    public void render(Graphics2D g) {
        if (this.inGame.isBuying()) {
            this.inBuy.render(g);
        } else {
            this.inGame.render(g);
        }
    }

    @Override
    protected void save(FileWriter file) throws IOException {
    }

    @Override
    protected void load(FileReader file) throws IOException {
        this.rh = new RessourcesHandler(this, "level1");
        this.shipFactory = new ShipFactory(this.rh, this);
        this.ship = this.shipFactory.createShip(ListShip.Gencore);
        this.entitiesLayer.setShip(this.ship);
        this.entityLayer1.setShip(this.ship);
        this.entityLayer2.setShip(this.ship);
        this.inGame.create(this.rh, this.ship);
        this.inBuy = new InBuy(this.inGame, this.rh, this.ship);
        this.inBuy.start();
    }

    @Override
    protected void save(XMLNode world) {
    }

    @Override
    protected void load(XMLNode world) {
    }
}
