package com.b3dgs.tyrian.bonus;

import com.b3dgs.lionengine.game.Camera;
import com.b3dgs.lionengine.input.Keyboard;
import com.b3dgs.lionengine.input.Mouse;
import com.b3dgs.tyrian.ListRessources;
import com.b3dgs.tyrian.RessourcesHandler;
import com.b3dgs.tyrian.SoundManager;
import com.b3dgs.tyrian.gameplay.AbstractBonus;
import com.b3dgs.tyrian.gameplay.Ship;
import java.awt.Graphics2D;

public abstract class AbstractBonusWeapon<ListWeapon extends Enum<?>> extends AbstractBonus {

    protected final RessourcesHandler rh;
    public final ListWeapon weapon;
    private float effect, offset;

    public AbstractBonusWeapon(RessourcesHandler rh, ListWeapon weapon, int id) {
        super(rh.getTiledSprite(ListRessources.Bonus3), id);
        this.rh = rh;
        this.weapon = weapon;
    }

    @Override
    public void update(Keyboard keyboard, Mouse mouse, float extrp) {
        super.update(keyboard, mouse, extrp);
        this.effect = (this.effect + 4.0f) % 360;
        this.offset = (float) Math.sin(Math.toRadians(this.effect)) * 12.0f;
        this.updateCollision((int) this.offset + 4, 4, 16, 20);
    }

    @Override
    public void render(Graphics2D g, Camera camera) {
        this.surface.render(g, this.tile + (int) this.frame, (int) (this.x + this.offset) - camera.getX(), camera.getY() - (int) this.y);
    }

    @Override
    public void giveBonus(Ship ship) {
        if (!ship.life.isEmpty()) {
            this.setShipWeapon(ship);
            SoundManager.play(SoundManager.SFX.POWERUP);
        }
    }

    protected abstract void setShipWeapon(Ship ship);
}
