package com.b3dgs.tyrian.gameplay;

import com.b3dgs.lionengine.Drawable;
import com.b3dgs.lionengine.drawable.Animator;
import com.b3dgs.lionengine.game.Camera;
import com.b3dgs.lionengine.game.shmup.AbstractShmupEntity;
import com.b3dgs.lionengine.game.shmup.EffectHandler;
import com.b3dgs.lionengine.input.Keyboard;
import com.b3dgs.lionengine.input.Mouse;
import com.b3dgs.lionengine.utility.Maths;
import com.b3dgs.tyrian.ListRessources;
import com.b3dgs.tyrian.RessourcesHandler;
import com.b3dgs.tyrian.effect.Explode;
import java.awt.Color;
import java.awt.Graphics2D;

public abstract class AbstractLayerEntity extends AbstractShmupEntity {
    
    protected final RessourcesHandler rh;
    protected final EffectHandler handler;
    protected final int tile;
    protected final Animator anim;
    protected final int cx, cy, cw, ch;
    protected Ship ship;
    
    public AbstractLayerEntity(RessourcesHandler rh, EffectHandler handler, int id, int tile, int cx, int cy, int cw, int ch) {
        super(rh.getTiledSprite(ListRessources.valueOf("EntityLayer" + (id + 1))));
        this.rh = rh;
        this.life.setMax(2);
        this.life.fill();
        this.handler = handler;
        this.tile = tile;
        this.cx = cx;
        this.cy = cy;
        this.cw = cw;
        this.ch = ch;
        this.anim = Drawable.DRAWABLE.createAnimator();
        this.invertYAxis(true);
    }
    
    public void setShip(Ship ship) {
        this.ship = ship;
    }
    
    @Override
    public void update(Keyboard keyboard, Mouse mouse, float extrp) {
        this.backupCoord();
        this.move(this.getVecX(), this.getVecY());
        this.updateCollision(this.cx, this.cy, this.cw, this.ch);
    }
    
    protected void renderFrame(Graphics2D g, Camera camera, int of, int ox, int oy) {
        this.surface.render(g, this.tile + of, (int) this.x + ox * 24 - camera.getX(), (int) Math.floor(camera.getY() - this.y + oy * 28));
        g.setColor(Color.red);
    }
    
    protected void explode(int minX, int maxX, int minY, int maxY, int sprite, int max) {
        for (int i = 0; i < max; i++) {
            Explode e = new Explode(this.rh.getTiledSprite(ListRessources.valueOf("Explode" + (sprite + 1))));
            long delay = Maths.random(0, i * 30);
            e.start(this.getX() + Maths.random(minX, maxX), this.getY() - Maths.random(minY, maxY), delay);
            this.handler.addQueue(e);
        }
    }
    
    @Override
    public void destroy(int x, int y) {
        if (!this.toDelete() && this.life.isEmpty()) {
            this.explode(0, 0, 0, 0, 0, 0);
        }
        this.destroy();
    }
}
