package com.b3dgs.tyrian.gameplay;

import com.b3dgs.lionengine.Drawable;
import com.b3dgs.lionengine.core.Alignment;
import com.b3dgs.lionengine.drawable.FontSprite;
import com.b3dgs.lionengine.drawable.Sprite;
import com.b3dgs.lionengine.drawable.Text;
import com.b3dgs.lionengine.input.Keyboard;
import com.b3dgs.lionengine.input.Mouse;
import com.b3dgs.lionengine.utility.Draws;
import com.b3dgs.lionengine.utility.Media;
import com.b3dgs.tyrian.ListRessources;
import com.b3dgs.tyrian.RessourcesHandler;
import com.b3dgs.tyrian.SoundManager;
import com.b3dgs.tyrian.equip.AbstractBattery;
import com.b3dgs.tyrian.equip.AbstractEquip;
import com.b3dgs.tyrian.equip.Battery1;
import com.b3dgs.tyrian.equip.Battery2;
import com.b3dgs.tyrian.equip.Battery3;
import com.b3dgs.tyrian.equip.Equip;
import com.b3dgs.tyrian.weapon.None;
import com.b3dgs.tyrian.weapon.Weapon;
import java.awt.Color;
import java.awt.Graphics2D;

public class InBuy {

    private static final Color BACK = new Color(96, 96, 96, 192);
    private static final Color OVER = new Color(128, 128, 128, 208);
    private static final Color HELP = new Color(207, 91, 0);
    private final Sprite background, transition, cursor;
    private final FontSprite font, font2;
    private final RessourcesHandler rh;
    private final Ship ship;
    private final InGame inGame;
    private final Text text;
    private final Weapon[] availFront, availRear;
    private final AbstractEquip[] battery;
    private float alpha;
    private int mx, my;
    private boolean in, out, click;
    private int selX, selY, selID;
    private Equip selEquip;

    public InBuy(InGame inGame, RessourcesHandler rh, Ship ship) {
        this.inGame = inGame;
        this.rh = rh;
        this.font = Drawable.DRAWABLE.createFontSprite(Media.get("misc", "font.png"), Media.get("misc", "fontdata.xml"), 8, 9);
        this.font.load(false);
        this.font2 = Drawable.DRAWABLE.createFontSprite(Media.get("misc", "font2.png"), Media.get("misc", "fontdata2.xml"), 6, 6);
        this.font2.load(false);
        this.availFront = new Weapon[3];
        this.availRear = new Weapon[3];
        this.battery = new AbstractEquip[3];
        this.battery[0] = new Battery1(rh);
        this.battery[1] = new Battery2(rh);
        this.battery[2] = new Battery3(rh);
        for (int i = 0; i < this.battery.length; i++) {
            if (ship.battery.energy == ((AbstractBattery) this.battery[i]).energy) {
                this.battery[i] = null;
            }
        }
        this.cursor = rh.getSprite(ListRessources.Cursor);
        this.background = rh.getSprite(ListRessources.Buy);
        this.transition = rh.getSprite(ListRessources.Transition);
        this.ship = ship;
        this.text = inGame.text;
        this.selID = -1;
    }

    public void start() {
        this.alpha = 255.0f;
        this.in = true;

        this.availFront[0] = this.rh.getRandomFrontWeapon();
        do {
            this.availFront[1] = this.rh.getRandomFrontWeapon();
        } while (this.availFront[1] == this.availFront[0]);
        do {
            this.availFront[2] = this.rh.getRandomFrontWeapon();
        } while (this.availFront[2] == this.availFront[1] || this.availFront[2] == this.availFront[0]);

        this.availRear[0] = this.rh.getRandomRearWeapon();
        do {
            this.availRear[1] = this.rh.getRandomRearWeapon();
        } while (this.availRear[1] == this.availRear[0]);
        do {
            this.availRear[2] = this.rh.getRandomRearWeapon();
        } while (this.availRear[2] == this.availRear[1] || this.availRear[2] == this.availRear[0]);
        this.selEquip = null;
        this.selID = -1;
    }

    public void update(Keyboard keyboard, Mouse mouse, float extrp) {
        this.mx = mouse.getOnWindowX();
        this.my = mouse.getOnWindowY();
        this.click = mouse.hasClickedOnce(Mouse.LEFT);
        if (this.in) {
            this.alpha -= 8.0;
            if (this.alpha < 0) {
                this.alpha = 0;
                this.in = false;
            }
            this.transition.setAlpha((int) this.alpha);
        }
        if (this.out) {
            this.alpha += 10.0;
            if (this.alpha > 255) {
                this.alpha = 255;
                this.out = false;
                this.inGame.bought();
            }
            this.transition.setAlpha((int) this.alpha);
        }
    }

    public void render(Graphics2D g) {
        this.background.render(g, 0, 0);
        this.ship.gfx.render(g, this.ship.gfxID, 8, 10);
        this.renderObject(g, this.ship.frontWeapon, 43, 12);
        this.renderObject(g, this.ship.rearWeapon, 43, 41);
        this.renderObject(g, this.ship.battery, 43, 70);
        if (this.selEquip != null) {
            this.description(g, this.selEquip.name(), this.selX + 24, this.selY);
        } else {
            g.setColor(HELP);
            g.drawLine(68, 26, 96, 15);
            g.drawLine(68, 55, 96, 15);
            g.drawLine(68, 84, 96, 15);
            this.font.draw(g, 102, 6, Alignment.LEFT, "This is your components", "Click on one of them...");
        }
        if (this.mx >= 110 && this.mx <= 210 && this.my >= 100 && this.my <= 126) {
            g.setColor(OVER);
            if (this.click) {
                this.out = true;
                SoundManager.play(SoundManager.SFX.ACCEPT);
            }
        } else {
            g.setColor(BACK);
        }
        g.fill3DRect(108, 100, 104, 26, true);
        this.font.draw(g, 160, 104, Alignment.CENTERED, "Exit and go", "to next wave");

        g.setColor(BACK);
        g.fill3DRect(143, 134, 175, 65, true);
        this.text.setColor(Color.LIGHT_GRAY);
        this.text.draw(g, "1) Click on one of your component.", 146, 138, Alignment.LEFT);
        this.text.draw(g, "2) Choose an action (sell, upgrade, buy)", 146, 149, Alignment.LEFT);
        this.text.draw(g, "3) When you are ready, click 'Exit...'", 146, 160, Alignment.LEFT);
        this.text.draw(g, "To reach this store, place your ship", 146, 177, Alignment.LEFT);
        this.text.draw(g, "under the big one when the wave ended.", 146, 188, Alignment.LEFT);

        this.cursor.render(g, this.mx, this.my);
        if (this.alpha > 0) {
            this.transition.render(g, 0, 0);
        }
    }

    private void renderObject(Graphics2D g, Equip equip, int x, int y) {
        if (this.mx >= x && this.mx <= x + 24 && this.my >= y && this.my <= y + 28) {
            if (this.click) {
                this.selX = x;
                this.selY = y;
                this.selID = equip.gfxID();
                this.selEquip = equip;
                SoundManager.play(SoundManager.SFX.CLICK);
            }
            g.setColor(OVER);
        } else {
            g.setColor(BACK);
        }
        g.fill3DRect(x, y, 24, 28, true);
        if (equip.gfxID() > -1) {
            equip.render(g, x, y);
        }
    }

    private void description(Graphics2D g, String desc, int x, int y) {
        this.font2.draw(g, 160, y - 3, Alignment.CENTERED, "CREDITS: " + this.ship.money.getCurrent());
        g.setColor(Color.RED);
        g.drawLine(x, y + 10, x + 10, y + 4);
        g.drawLine(x + 10, y + 4, 218, y + 4);
        g.setColor(BACK);
        g.fill3DRect(218, 4, 100, 124, true);
        this.text.setColor(Color.CYAN);
        if (this.selEquip instanceof Weapon) {
            Weapon w = (Weapon) this.selEquip;
            if (w.isFront()) {
                this.text.draw(g, desc, 268, 10, Alignment.CENTERED);
            } else {
                this.text.draw(g, desc, 268, 10, Alignment.CENTERED);
            }
        } else {
            this.text.draw(g, desc, 268, 10, Alignment.CENTERED);
        }

        if (this.selID > -1 && this.selEquip instanceof Weapon) {
            Weapon weapon = (Weapon) this.selEquip;
            g.setColor(Color.GRAY);
            int next = weapon.level.getCurrent();
            int upCost = weapon.price() * (next * next);
            if (this.mx >= 228 && this.mx <= 237 && this.my >= 38 && this.my <= 47) {
                g.setColor(Color.LIGHT_GRAY);
                if (this.click) {
                    if (weapon.level.isEnough(2)) {
                        this.ship.money.increase(upCost);
                        weapon.level.decrease(1);
                        SoundManager.play(SoundManager.SFX.CLICK);
                    } else {
                        SoundManager.play(SoundManager.SFX.MAX);
                    }
                }
            }
            this.text.setColor(Color.ORANGE);
            this.text.draw(g, "Upgrade weapon:", 270, 24, Alignment.CENTERED);

            this.text.setColor(Color.WHITE);
            g.fill3DRect(228, 36, 9, 9, true);
            this.text.draw(g, "-", 231, 36, Alignment.LEFT);

            g.setColor(Color.GRAY);
            next = weapon.level.getCurrent() + 1;
            upCost = weapon.price() * (next * next);
            if (this.mx >= 300 && this.mx <= 309 && this.my >= 36 && this.my <= 45) {
                g.setColor(Color.LIGHT_GRAY);
                if (this.click) {
                    if (!weapon.level.isFull() && this.ship.money.isEnough(upCost)) {
                        this.ship.money.decrease(upCost);
                        weapon.level.increase(1);
                        SoundManager.play(SoundManager.SFX.CLICK);
                    } else {
                        SoundManager.play(SoundManager.SFX.MAX);
                    }
                }
            }
            g.fill3DRect(300, 36, 9, 9, true);
            this.text.draw(g, "+", 302, 36, Alignment.LEFT);

            Draws.percentBar(g, 238, 36, 60, 9, weapon.level.getPercent(), 100, 1, 1, Color.RED, Color.YELLOW, Color.GRAY, false, true, true);

            if (this.ship.money.isEnough(upCost)) {
                this.text.setColor(Color.GREEN);
            } else {
                this.text.setColor(Color.RED);
            }
            this.text.draw(g, "Cost: " + upCost, 268, 48, Alignment.CENTERED);
        }

        Equip[] list = null;
        if (this.selEquip instanceof Weapon) {
            Weapon weapon = (Weapon) this.selEquip;
            this.text.setColor(Color.ORANGE);
            this.text.draw(g, "Buy new weapon:", 270, 66, Alignment.CENTERED);

            if (weapon.isFront()) {
                list = this.availFront;
            } else {
                list = this.availRear;
            }
        } else if (this.selEquip instanceof AbstractBattery) {
            list = this.battery;
        }

        this.renderBuyEquip(g, list[0], 0, 228, 78);
        this.renderBuyEquip(g, list[1], 1, 258, 78);
        this.renderBuyEquip(g, list[2], 2, 288, 78);

        if ((this.selEquip instanceof AbstractBattery)) {
            return;
        }
        int sold = 0;
        Weapon weapon = null;
        if (this.selEquip != null) {
            if (this.selEquip instanceof Weapon) {
                weapon = (Weapon) this.selEquip;
                for (int i = 2; i < weapon.level.getCurrent() + 1; i++) {
                    sold += (weapon.price() * (i * i));
                }
                sold += weapon.price() / 2;
            } else {
                sold = this.selEquip.price() / 2;
            }
            if (this.mx >= 110 && this.mx <= 210 && this.my >= 80 && this.my <= 98) {
                g.setColor(OVER);
                if (this.click) {
                    if (weapon != null) {
                        if (weapon.isFront()) {
                            this.ship.setWeaponFront(this.rh.getWeaponFront(ListWeaponFront.NoneFront));
                        } else {
                            this.ship.setWeaponRear(this.rh.getWeaponRear(ListWeaponRear.NoneRear));
                        }
                    }
                    this.ship.money.increase(sold);
                    this.selEquip = null;
                    this.selID = -1;
                    SoundManager.play(SoundManager.SFX.CLICK);
                }
            } else {
                g.setColor(BACK);
            }
            this.text.setColor(Color.PINK);
            if (this.selEquip != null && this.selEquip.gfxID() > -1) {
                g.fill3DRect(116, 80, 86, 18, true);
                this.font.draw(g, 160, 84, Alignment.CENTERED, "Sell: " + sold);
            }
        }
    }

    private void renderBuyEquip(Graphics2D g, Equip equip, int id, int x, int y) {
        if (equip == null || equip instanceof None) {
            return;
        }
        Weapon weapon = null;
        if (equip instanceof Weapon) {
            weapon = (Weapon) equip;
        }
        if (this.mx >= x && this.mx <= x + 24 && this.my >= y && this.my <= y + 28) {
            if (this.click && this.ship.money.isEnough(equip.price())) {
                if (weapon != null) {
                    int sold = 0;
                    if (weapon.isFront()) {
                        for (int i = 2; i < this.ship.frontWeapon.level.getCurrent() + 1; i++) {
                            sold += (this.ship.frontWeapon.price() * (i * i));
                        }
                        sold += this.ship.frontWeapon.price() / 2;
                        this.ship.setWeaponFront(weapon);
                        do {
                            this.availFront[id] = this.rh.getRandomFrontWeapon();
                        } while (this.availFront[id] == weapon);
                    } else {
                        for (int i = 2; i < this.ship.rearWeapon.level.getCurrent() + 1; i++) {
                            sold += (this.ship.rearWeapon.price() * (i * i));
                        }
                        sold += this.ship.frontWeapon.price() / 2;
                        this.ship.setWeaponRear(weapon);
                        do {
                            this.availRear[id] = this.rh.getRandomRearWeapon();
                        } while (this.availRear[id] == weapon);
                    }
                    this.ship.money.increase(sold);
                } else if (equip instanceof AbstractBattery) {
                    this.ship.setBattery((AbstractBattery) equip);
                    for (int i = 0; i < this.battery.length; i++) {
                        if (this.battery[i] != null && this.ship.battery.energy == ((AbstractBattery) this.battery[i]).energy) {
                            this.battery[i] = null;
                        }
                    }
                }
                this.ship.money.decrease(equip.price());
                this.selEquip = equip;
                this.selID = equip.gfxID();
                SoundManager.play(SoundManager.SFX.CLICK);
            }
            g.setColor(OVER);
        } else {
            g.setColor(BACK);
        }
        g.fill3DRect(x, y, 24, 28, true);
        equip.render(g, x, y);
        this.text.setColor(Color.YELLOW);
        this.text.draw(g, "Price:", x + 12, y + 30, Alignment.CENTERED);
        if (this.ship.money.isEnough(equip.price())) {
            this.text.setColor(Color.GREEN);
        } else {
            this.text.setColor(Color.RED);
        }
        this.text.draw(g, "" + equip.price(), x + 12, y + 40, Alignment.CENTERED);
    }
}
