package com.b3dgs.tyrian.gameplay;

import com.b3dgs.lionengine.Audio;
import com.b3dgs.lionengine.Drawable;
import com.b3dgs.lionengine.File;
import com.b3dgs.lionengine.audio.Midi;
import com.b3dgs.lionengine.core.Alignment;
import com.b3dgs.lionengine.drawable.Text;
import com.b3dgs.lionengine.file.FileReader;
import com.b3dgs.lionengine.file.FileWriter;
import com.b3dgs.lionengine.game.Camera;
import com.b3dgs.lionengine.game.map.DefaultTile;
import com.b3dgs.lionengine.game.shmup.AbstractShmupEntity;
import com.b3dgs.lionengine.game.shmup.EffectHandler;
import com.b3dgs.lionengine.input.Keyboard;
import com.b3dgs.lionengine.input.Mouse;
import com.b3dgs.lionengine.utility.Displays;
import com.b3dgs.lionengine.utility.LevelRipConverter;
import com.b3dgs.lionengine.utility.Maths;
import com.b3dgs.lionengine.utility.Media;
import com.b3dgs.lionengine.utility.Timer;
import com.b3dgs.tyrian.Background;
import com.b3dgs.tyrian.BonusFactory;
import com.b3dgs.tyrian.EntitiesHandler;
import com.b3dgs.tyrian.Entity1Handler;
import com.b3dgs.tyrian.Entity2Handler;
import com.b3dgs.tyrian.ListRessources;
import com.b3dgs.tyrian.Map;
import com.b3dgs.tyrian.RessourcesHandler;
import com.b3dgs.tyrian.entity.layer2.level1.Armory;
import com.b3dgs.tyrian.entity.layer2.level1.EntityLayer2Factory;
import com.b3dgs.tyrian.entity.layer2.level1.ListEntityLayer2Level1;
import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.event.KeyEvent;
import java.io.IOException;

public class InGame {

    private class Data {

        private final int m1, m2, m3, time, mapMin, mapMax, musMax;

        private Data(int m1, int m2, int m3, int time, int mapMin, int mapMax, int musMax) {
            this.m1 = m1;
            this.m2 = m2;
            this.m3 = m3;
            this.time = time;
            this.mapMin = mapMin;
            this.mapMax = mapMax;
            this.musMax = musMax;
        }
    }
    private static final Color PAUSE = new Color(128, 128, 128, 128);
    public final Text text;
    private final long waveTime = 60000L;
    private final int maps = 20;
    private final Camera camLayer1, camLayer2;
    private final EntitiesHandler entitiesHandler;
    private final Entity1Handler entityLayer1;
    private final Entity2Handler entityLayer2;
    private final EffectHandler effects;
    private final Timer spawn, timedLevel, interTime;
    private Midi midi;
    private EntityLayer2Factory layer2Factory;
    private BonusFactory bonusFactory;
    private RessourcesHandler rh;
    private Map map, map2, map3;
    private Ship ship;
    private Armory armory;
    private Background background;
    private float layer1;
    private boolean load1, load2, load3;
    private boolean buy, inter, inWave, appeared, firstBought, gameover, paused, firstPaused;
    private int level, interWait, lastMusic;
    private final Data[] levels = {
        new Data(0, 0, 0, 0, 0, 0, 0),
        new Data(100, 300, 10000, 550, 0, 10, 2),
        new Data(80, 150, 5000, 475, 0, 12, 3),
        new Data(60, 125, 2500, 400, 0, 13, 4),
        new Data(50, 90, 1000, 350, 0, 15, 5),
        new Data(55, 85, 500, 275, 0, 17, 5),
        new Data(70, 80, 250, 200, 0, 18, 5),
        new Data(90, 60, 150, 150, 0, 19, 5),
        new Data(100, 70, 75, 100, 0, 20, 5),
        new Data(120, 85, 50, 75, 1, 20, 5),
        new Data(150, 100, 40, 50, 14, 20, 5)
    };

    public InGame(Entity1Handler entityLayer1, Entity2Handler entityLayer2, EntitiesHandler handler, EffectHandler effects) {
        this.text = Drawable.DRAWABLE.createText(Font.DIALOG, 9, Text.NORMAL);
        this.entitiesHandler = handler;
        this.entityLayer1 = entityLayer1;
        this.entityLayer2 = entityLayer2;
        this.effects = effects;
        this.camLayer1 = new Camera(0, 0);
        this.camLayer2 = new Camera(0, 0);
        this.spawn = new Timer();
        this.timedLevel = new Timer();
        this.interTime = new Timer();
        this.layer1 = 0.0f;
        this.level = 0;
        this.lastMusic = -1;
    }

    public void create(RessourcesHandler rh, Ship ship) throws IOException {
        this.rh = rh;
        this.layer2Factory = new EntityLayer2Factory(this.rh, this.effects);
        this.bonusFactory = new BonusFactory(this.rh, this.entitiesHandler);
        this.background = new Background(this.rh);
        this.ship = ship;
        //for (int i = 0; i <= this.maps; i++) {
        //    this.importLevel(i + ".png", i + ".map");
        //}
        //System.exit(0);
        this.armory = new Armory(rh, this.effects);
        this.map = new Map(this.rh, this.entityLayer1, this.entitiesHandler, this.effects);
        this.map2 = new Map(this.rh, this.entityLayer1, this.entitiesHandler, this.effects);
        this.map3 = new Map(this.rh, this.entityLayer1, this.entitiesHandler, this.effects);
        this.map.setOffsetY(0);
        this.map.load(File.FILE.createFileReader(Media.get("levels", Maths.random(0, 1) + ".map")));
        this.load1 = true;
        this.load2 = false;
        this.buy = true;
        this.inter = true;
        this.inWave = false;
        this.appeared = false;
        this.firstBought = true;
        this.level = 0;
        this.paused = false;
        this.music("buy", true);
        this.timedLevel.start();
    }

    public void pause() {
        this.timedLevel.pause();
        this.interTime.pause();
        this.midi.pause();
        this.paused = true;
        this.firstPaused = false;
    }

    public void unpause() {
        this.timedLevel.unpause();
        this.interTime.unpause();
        this.midi.unpause();
        this.paused = false;
    }

    public void update(Keyboard keyboard, Mouse mouse, float extrp) {
        if (keyboard.isPressedOnce(KeyEvent.VK_P)) {
            if (!this.paused) {
                this.pause();
            } else {
                this.unpause();
            }
        }
        if (this.paused) {
            return;
        }
        this.ship.update(keyboard, mouse, extrp);
        if (!this.firstBought) {
            this.armory.update(keyboard, mouse, extrp);

            if (this.armory.canBuy() && this.armory.collide(this.ship)) {
                this.armory.bought();
                this.music("buy", true);
                this.buy = true;
            }
        }

        this.entityLayer2.update(keyboard, mouse, extrp);
        this.entityLayer1.setOffsetX(this.camLayer2.getX() - this.camLayer1.getX());
        this.entityLayer1.skipMove(!this.inWave);
        this.entityLayer1.update(keyboard, mouse, extrp);
        this.entitiesHandler.update(keyboard, mouse, extrp);
        this.effects.update(keyboard, mouse, extrp);
        this.background.update(extrp);
        this.camLayer1.place(this.ship.getXReal() * 0.8f, this.ship.getY());
        this.camLayer2.place(this.ship.getX(), this.ship.getY());

        if (this.inWave) {
            if (!this.timedLevel.elapsed(this.waveTime - 3000)) {
                if (!this.armory.canBuy() && this.spawn.elapsed(this.levels[this.level].time)) {
                    if (Maths.random(0, this.levels[this.level].m1) == 0) {
                        this.add(this.layer2Factory.createEntity(ListEntityLayer2Level1.MeteorSmall), 0.0f, -Maths.random(14, 18) / 16.0f);
                    }
                    if (Maths.random(0, this.levels[this.level].m2) == 0) {
                        this.add(this.layer2Factory.createEntity(ListEntityLayer2Level1.MeteorMedium), 0.0f, -Maths.random(14, 18) / 16.0f);
                    }
                    if (Maths.random(0, this.levels[this.level].m3) == 0) {
                        this.add(this.layer2Factory.createEntity(ListEntityLayer2Level1.MeteorLarge), 0.0f, -Maths.random(14, 18) / 16.0f);
                    }
                }
                if (Maths.random(0, 35) == 0) {
                    this.add(this.bonusFactory.createBonusCoin(this.bonusFactory.getRndCoin()), 0.0f, -1.5f);
                }
            }

            if (this.timedLevel.elapsed(this.waveTime)) {
                if (!this.inter) {
                    this.inter = true;
                    this.inWave = false;
                    this.music("end", false);
                    this.interTime.start();
                    this.interWait = 5000;
                }
            }
            this.layer1 += 0.75f;
        }

        if (this.inter) {
            if (!this.appeared && this.interTime.elapsed(2000)) {
                if (!this.firstBought) {
                    this.armory.appear();
                }
                this.appeared = true;
            }
            if (this.interTime.elapsed(this.interWait) && !this.buy) {
                this.level = Maths.fixBetween(this.level + 1, 0, this.levels.length - 1);
                this.inter = false;
                this.inWave = true;
                this.firstBought = false;
                this.appeared = false;
                int rnd = Maths.random(0, this.levels[this.level].musMax);
                if (rnd == this.lastMusic) {
                    rnd = (rnd + 1) % 5;
                }
                this.lastMusic = rnd;
                this.music("music" + rnd, false);
                this.ship.earth.increase(15);
                this.background.effect();
                this.spawn.start();
                this.timedLevel.start();
            }
        }

        int y = ((int) this.layer1) % (280 * 3);

        try {
            int min = this.levels[this.level].mapMin;
            int max = this.levels[this.level].mapMax;
            if (!this.load1 && y >= 280 * 2) {
                this.map.setOffsetY(280);
                FileReader fr = File.FILE.createFileReader(Media.get("levels", Maths.random(min, max) + ".map"));
                this.map.load(fr);
                fr.close();
                this.load1 = true;
                this.load2 = false;
            }
            if (!this.load2 && y >= 0 && y < 280) {
                this.map2.setOffsetY(280);
                FileReader fr = File.FILE.createFileReader(Media.get("levels", Maths.random(min, max) + ".map"));
                this.map2.load(fr);
                fr.close();
                this.load2 = true;
                this.load3 = false;
            }
            if (!this.load3 && y >= 280 && y < 280 * 2) {
                this.map3.setOffsetY(280);
                FileReader fr = File.FILE.createFileReader(Media.get("levels", Maths.random(min, max) + ".map"));
                this.map3.load(fr);
                fr.close();
                this.load3 = true;
                this.load1 = false;
            }
        } catch (Exception ex) {
        }

        if (!this.gameover && this.ship.life.isEmpty()) {
            this.gameover = true;
            this.music("gameover", false);
        }
    }

    public void render(Graphics2D g) {
        if (this.paused && this.firstPaused) {
            return;
        }
        this.background.render(g, this.camLayer1);
        int y = ((int) this.layer1) % (280 * 3);
        if (y >= 0 && y < 280 * 2) {
            this.map.render(g, this.camLayer1.getX(), y, 11, 8);
        }
        if (y >= 280 && y < 280 * 3) {
            this.map2.render(g, this.camLayer1.getX(), y - 280, 11, 8);
        }
        if (y >= 280 * 2 && y < 280 * 3) {
            this.map3.render(g, this.camLayer1.getX(), y - 280 * 2, 11, 8);
        }
        if (y < 280) {
            this.map3.render(g, this.camLayer1.getX(), y + 280, 11, 80);
        }
        this.entityLayer1.render(g, this.camLayer1);
        this.entityLayer2.render(g, this.camLayer2);
        this.entitiesHandler.render(g, this.camLayer2);
        this.effects.render(g, this.camLayer2);
        this.ship.render(g, this.camLayer2);
        this.armory.render(g, this.camLayer2);
        this.ship.renderHud(g);
        this.text.setColor(Color.LIGHT_GRAY);
        this.text.draw(g, "Credits: " + this.ship.money.getCurrent(), 17, 189, Alignment.LEFT);
        this.text.draw(g, "Level: " + (this.level), 248, 189, Alignment.RIGHT);
        if (this.inWave) {
            this.text.draw(g, "Wave Time Left: " + (60 - this.timedLevel.elapsed() / 1000), 133, 4, Alignment.CENTERED);
        } else {
            this.text.draw(g, "Next wave in: " + ((this.interWait + 1000 - this.interTime.elapsed()) / 1000), 133, 4, Alignment.CENTERED);
        }
        if (this.paused) {
            if (!this.firstPaused) {
                this.rh.getSprite(ListRessources.Paused).render(g, 93, 92);
                g.setColor(PAUSE);
                g.fillRect(0, 0, 320, 200);
                this.firstPaused = true;
            }
        }
    }

    private void add(AbstractShmupEntity entity, float vx, float vy) {
        entity.place(Maths.random(0, 260 - entity.getWidth()), this.ship.getY() + entity.getHeight() + 64);
        entity.setVect(vx, vy);
        entity.backupCoord();
        this.entitiesHandler.add(entity);
    }

    private void add(AbstractLayerEntity entity, float vx, float vy) {
        entity.place(Maths.random(0, 260 - entity.getWidth()), this.ship.getY() + entity.getHeight() + 64);
        entity.setVect(vx, vy);
        entity.backupCoord();
        this.entityLayer2.add(entity);
        this.spawn.start();
    }

    private void importLevel(String levelrip, String out) {
        LevelRipConverter<DefaultTile> lrc = new LevelRipConverter<DefaultTile>();
        Map m = new Map(this.rh, this.entityLayer1, this.entitiesHandler, this.effects);
        lrc.start(Media.get("levels", "images", levelrip), m, "tiles", "level1", true);
        try {
            FileWriter fw = File.FILE.createFileWriter(Media.get("levels", out));
            m.save(fw);
            fw.close();
        } catch (IOException ex) {
            Displays.error("importLevel", "Error during map import on: " + levelrip);
        }
    }

    public void bought() {
        this.buy = false;
        this.music("next", false);
        this.interTime.start();
        this.interWait = 3000;
    }

    public boolean isBuying() {
        return this.buy;
    }

    public void music(String name, boolean repeat) {
        this.stopMusic();
        this.midi = Audio.AUDIO.loadMidi(Media.get("musics", name + ".mid"));
        this.midi.play(repeat);
        this.midi.setVolume(85);
    }

    public void stopMusic() {
        if (this.midi != null) {
            this.midi.stop();
        }
    }
}
