package com.b3dgs.tyrian.gameplay;

import com.b3dgs.lionengine.drawable.Sprite;
import com.b3dgs.lionengine.drawable.TiledSprite;
import com.b3dgs.lionengine.game.Alterable;
import com.b3dgs.lionengine.game.Camera;
import com.b3dgs.lionengine.game.shmup.AbstractShip;
import com.b3dgs.lionengine.game.shmup.AbstractShmupEntity;
import com.b3dgs.lionengine.game.shmup.EffectHandler;
import com.b3dgs.lionengine.input.Keyboard;
import com.b3dgs.lionengine.input.Mouse;
import com.b3dgs.lionengine.utility.Draws;
import com.b3dgs.lionengine.utility.Maths;
import com.b3dgs.lionengine.utility.Timer;
import com.b3dgs.tyrian.ListRessources;
import com.b3dgs.tyrian.RessourcesHandler;
import com.b3dgs.tyrian.SoundManager;
import com.b3dgs.tyrian.effect.Explode;
import com.b3dgs.tyrian.equip.AbstractBattery;
import com.b3dgs.tyrian.equip.Battery1;
import com.b3dgs.tyrian.weapon.Weapon;
import java.awt.Color;
import java.awt.Graphics2D;

public class Ship extends AbstractShip {

    public final Sprite hit, hud, gameover;
    public final TiledSprite gfx;
    public final int gfxID;
    public final Alterable energy, shield, money, earth;
    public Weapon frontWeapon, rearWeapon;
    public AbstractBattery battery;
    private final RessourcesHandler rh;
    private final EffectHandler effects;
    private final Timer hitTimer, shieldCanRegenTimer, shieldRegenTimer, gameoverTimer;
    private int gameoverAlpha;
    private boolean isHit, gameoverStarted;

    public Ship(RessourcesHandler rh, int gfxID, int idleFrame, int maxAngle, EffectHandler effects) {
        super(rh.getTiledSprite(ListRessources.Ships), idleFrame, maxAngle);
        this.rh = rh;
        this.gfx = rh.getTiledSprite(ListRessources.ShipUpgrade);
        this.gameover = rh.getSprite(ListRessources.Gameover);
        this.gameoverAlpha = 0;
        this.gfxID = gfxID;
        this.effects = effects;
        this.shield = new Alterable(50);
        this.energy = new Alterable(1000);
        this.earth = new Alterable(100);
        this.money = new Alterable(Integer.MAX_VALUE);
        this.dmg.setMin(10);
        this.dmg.setMax(10);
        this.borderMinX = 0;
        this.borderMaxX = 266 - this.getWidth();
        this.borderMinY = 0;
        this.borderMaxY = 184 - this.getHeight();
        this.hit = rh.getSprite(ListRessources.Hit);
        this.hit.load(false);
        this.hud = rh.getSprite(ListRessources.Hud);
        this.hud.load(false);
        this.hitTimer = new Timer();
        this.shieldCanRegenTimer = new Timer();
        this.shieldRegenTimer = new Timer();
        this.gameoverTimer = new Timer();
        this.frontWeapon = rh.getWeaponFront(ListWeaponFront.PulseCannon);
        this.rearWeapon = rh.getWeaponRear(ListWeaponRear.NoneRear);
        this.battery = new Battery1(rh);
        this.life.setMax(50);
        this.life.fill();
        this.shield.fill();
        this.money.set(5000);
        this.isHit = false;
        this.invertYAxis(true);
        this.placeOffset(121, 160);
        this.backupCoord();
    }

    @Override
    public void updateInput(Keyboard keyboard, Mouse mouse) {
        this.left = keyboard.isPressed(Keyboard.LEFT);
        this.right = keyboard.isPressed(Keyboard.RIGHT);
        this.up = keyboard.isPressed(Keyboard.UP);
        this.down = keyboard.isPressed(Keyboard.DOWN);
        this.fire1 = mouse.hasClicked(Mouse.LEFT) || keyboard.isPressed(Keyboard.SPACE);
        this.fire2 = mouse.hasClicked(Mouse.RIGHT);
    }

    @Override
    public void update(Keyboard keyboard, Mouse mouse, float extrp) {
        if (!this.life.isEmpty()) {
            if (keyboard.used()) {
                this.setUseMouse(false);
            }
            if (mouse.moved()) {
                this.setUseMouse(true);
            }
            super.update(keyboard, mouse, extrp);
            this.setX(this.getXOffset() / 7);

            if (this.fire1) {
                if (this.energy.isEnough(this.frontWeapon.getEnergy()) && this.frontWeapon.start(this)) {
                    this.energy.decrease(this.frontWeapon.getEnergy());
                }
                if (this.energy.isEnough(this.rearWeapon.getEnergy()) && this.rearWeapon.start(this)) {
                    this.energy.decrease(this.rearWeapon.getEnergy());
                }
            }
            this.frontWeapon.update(this, keyboard, mouse, extrp);
            this.rearWeapon.update(this, keyboard, mouse, extrp);
            this.energy.increase(this.battery.energy);
            if (!this.shield.isFull() && this.shieldCanRegenTimer.elapsed(1000)) {
                if (this.energy.isEnough(100) && this.shieldRegenTimer.elapsed(1000)) {
                    this.shield.increase(10);
                    this.energy.decrease(100);
                    this.shieldRegenTimer.start();
                }
            }
            if (this.earth.isEmpty()) {
                this.die();
            }
            this.updateCollision((int) this.getXOffset(), (int) this.getYOffset(), 24, 24);
        } else {
            if (!this.gameoverStarted) {
                this.gameoverTimer.start();
                this.gameoverStarted = true;
            }
        }
    }

    @Override
    public void renderWeapon(Graphics2D g, Camera camera) {
        this.frontWeapon.render(g, camera);
        this.rearWeapon.render(g, camera);
    }

    @Override
    public void render(Graphics2D g, Camera camera) {
        if (!this.life.isEmpty()) {
            super.render(g, camera);
        }
        if (!this.hitTimer.elapsed(50)) {
            this.hit.render(g, this.getX() + (int) this.getXOffset() + this.width / 2 - this.hit.getWidth() / 2 - camera.getX(),
                    camera.getY() - (this.getY() - (int) this.getYOffset() + this.height / 2 - this.hit.getHeight() / 2));
        } else {
            this.isHit = false;
        }
        if (this.gameoverStarted && this.gameoverTimer.elapsed(2000)) {
            this.gameoverAlpha = Maths.fixBetween(this.gameoverAlpha + 4, 0, 255);
            this.gameover.setAlpha(this.gameoverAlpha);
            this.gameover.render(g, 80, 85);
        }
    }

    public void renderHud(Graphics2D g) {
        this.hud.render(g, 0, 0);
        Draws.percentBar(g, 270, 136, 9, 60, 100, this.shield.getPercent(), 0, 0, Color.BLUE, Color.BLUE, null, false, true, true);
        Draws.percentBar(g, 307, 136, 9, 60, 100, this.life.getPercent(), 0, 0, Color.ORANGE, Color.ORANGE, null, false, true, true);
        Draws.percentBar(g, 268, 11, 11, 96, 100, this.energy.getPercent(), 0, 0, Color.YELLOW, Color.RED, null, false, true, true);
        Draws.percentBar(g, 267, 115, 49, 12, this.earth.getPercent(), 100, 0, 0, Color.GREEN, Color.RED, null, false, false, true);

        Draws.percentBar(g, 285, 17, 26, 3, this.frontWeapon.level.getPercent(), 100, 0, 0, Color.RED, Color.RED, null, false, true, false);
        Draws.percentBar(g, 285, 38, 26, 3, this.rearWeapon.level.getPercent(), 100, 0, 0, Color.RED, Color.RED, null, false, true, false);
    }

    @Override
    public void hitBy(AbstractShmupEntity entity) {
        if (entity instanceof AbstractBonus) {
            entity.hitBy(this);
        } else if (!this.life.isEmpty()) {
            if (this.hitTimer.elapsed(50)) {
                this.setHitForce(-entity.getVecY());
                if (this.shield.isEnough(5)) {
                    this.shield.decrease(5);
                    SoundManager.play(SoundManager.SFX.PLAYER_SHIELD);
                }
                if (this.shield.getPercent() == 0) {
                    this.life.decrease(5);
                    SoundManager.play(SoundManager.SFX.PLAYER_LIFE);
                }
                if (this.life.isEmpty()) {
                    this.die();
                }
                this.shieldCanRegenTimer.start();
                entity.hitBy(this);
                if (!this.isHit) {
                    this.hitTimer.start();
                    this.isHit = true;
                }
            }
        }
    }

    private void die() {
        this.life.set(0);
        this.explode((int) this.getXOffset() - 24, (int) this.getXOffset() + 24, (int) this.getYOffset() - 28, (int) this.getYOffset() + 28, 0, 112);
        this.explode((int) this.getXOffset() - 4, (int) this.getXOffset() + 4, (int) this.getYOffset() - 8, (int) this.getYOffset() + 8, 0, 64);
        this.gameoverTimer.start();
    }

    protected void explode(int minX, int maxX, int minY, int maxY, int sprite, int max) {
        for (int i = 0; i < max; i++) {
            Explode e = new Explode(this.rh.getTiledSprite(ListRessources.valueOf("Explode" + (sprite + 1))));
            long delay = Maths.random(0, i * 25);
            e.start(this.getX() + Maths.random(minX, maxX), this.getY() - Maths.random(minY, maxY), delay);
            this.effects.addQueue(e);
        }
    }

    public void setWeaponFront(Weapon weapon) {
        this.frontWeapon = weapon;
    }

    public void setWeaponRear(Weapon weapon) {
        this.rearWeapon = weapon;
    }

    public void setBattery(AbstractBattery battery) {
        this.battery = battery;
    }
}
