package com.b3dgs.tyrian.weapon.front;

import com.b3dgs.tyrian.RessourcesHandler;
import com.b3dgs.tyrian.SoundManager;
import com.b3dgs.tyrian.gameplay.Ship;
import com.b3dgs.tyrian.weapon.Weapon;

public class SonicWaveFront extends Weapon {

    private static final int FRAME = 9;
    private int smaWave = 0, medWave = 0, bigWave = 0;

    public SonicWaveFront(RessourcesHandler rh) {
        super(rh, "Front Sonic Wave", 15, 750);
    }

    private void lvl1Top(Ship ship, int dmg, int of, int ox) {
        this.addBullet(ship, this.c, dmg, FRAME + of, -0.5f, 0.2f, ox, -6);
        this.addBullet(ship, this.c, dmg, FRAME + of, 0.5f, 0.2f, ox, -6);
    }

    private void lvl2Top(Ship ship, int dmg, int of) {
        this.addBullet(ship, this.c, dmg, FRAME + 1 + of, -1.0f, 0.25f, -8, -6);
        this.addBullet(ship, this.c, dmg, FRAME + 1 + of, 1.0f, 0.25f, 8, -6);
    }

    private void smallWave(Ship ship, int dmg, float vecX, float vecY, float offX, float offY) {
        this.addBullet(ship, this.c, this.smaWave, dmg, FRAME + 193, vecX, vecY, offX - 6, offY);
        this.addBullet(ship, this.c, this.smaWave, dmg, FRAME + 194, vecX, vecY, offX + 6, offY);
        this.smaWave++;
    }

    private void mediumWave(Ship ship, int dmg, float vecX, float vecY, float offX, float offY) {
        this.addBullet(ship, this.l, this.medWave, dmg, FRAME + 225, vecX, vecY, offX - 18, offY);
        this.addBullet(ship, this.l, this.medWave, dmg, FRAME + 226, vecX, vecY, offX - 6, offY);
        this.addBullet(ship, this.l, this.medWave, dmg, FRAME + 227, vecX, vecY, offX + 6, offY);
        this.addBullet(ship, this.l, this.medWave, dmg, FRAME + 228, vecX, vecY, offX + 18, offY);
        this.medWave++;
    }

    private void bigWave(Ship ship, int dmg, float vecX, float vecY, float offX, float offY) {
        this.addBullet(ship, this.l, this.bigWave, dmg, FRAME + 195, vecX, vecY, offX - 18, offY - 6);
        this.addBullet(ship, this.l, this.bigWave, dmg, FRAME + 196, vecX, vecY, offX - 6, offY - 6);
        this.addBullet(ship, this.l, this.bigWave, dmg, FRAME + 197, vecX, vecY, offX + 6, offY - 6);
        this.addBullet(ship, this.l, this.bigWave, dmg, FRAME + 198, vecX, vecY, offX + 18, offY - 6);
        this.addBullet(ship, this.l, this.bigWave, dmg, FRAME + 210, vecX, vecY, offX - 18, offY + 6);
        this.addBullet(ship, this.l, this.bigWave, dmg, FRAME + 211, vecX, vecY, offX - 6, offY + 6);
        this.addBullet(ship, this.l, this.bigWave, dmg, FRAME + 212, vecX, vecY, offX + 6, offY + 6);
        this.addBullet(ship, this.l, this.bigWave, dmg, FRAME + 213, vecX, vecY, offX + 18, offY + 6);
        this.bigWave++;
    }

    @Override
    protected void fire(Ship ship) {
        int dmg;
        this.setRate(180);
        this.setCollision(0, 1, 0, 1);
        switch (this.level.getCurrent()) {
            case 1:
                dmg = 15;
                this.setEnergy(40);
                this.lvl1Top(ship, dmg, 0, 0);
                break;
            case 2:
                dmg = 18;
                this.setEnergy(50);
                this.lvl1Top(ship, dmg, 0, 0);
                this.lvl2Top(ship, dmg, 0);
                break;
            case 3:
                dmg = 24;
                this.setEnergy(60);
                this.lvl1Top(ship, dmg, 2, 5);
                this.lvl1Top(ship, dmg, 2, -5);
                break;
            case 4:
                dmg = 30;
                this.setEnergy(70);
                this.smallWave(ship, dmg, -0.6f, 0.22f, 0, -6);
                this.smallWave(ship, dmg, 0.6f, 0.22f, 0, -6);
                break;
            case 5:
                dmg = 35;
                this.setEnergy(100);
                this.lvl1Top(ship, dmg, 2, 0);
                this.mediumWave(ship, dmg + 30, 0.0f, 6.0f, 0, 0);
                break;
            case 6:
                dmg = 35;
                this.setEnergy(150);
                this.setRate(250);
                this.lvl2Top(ship, dmg, 1);
                this.bigWave(ship, dmg + 75, 0, 3.0f, 0, 0);
                break;
        }
        SoundManager.play(SoundManager.SFX.WEAPON_WAVE);
    }
}
