package com.b3dgs.tyrian.weapon.rear;

import com.b3dgs.tyrian.RessourcesHandler;
import com.b3dgs.tyrian.SoundManager;
import com.b3dgs.tyrian.gameplay.Ship;
import com.b3dgs.tyrian.weapon.Weapon;

public class SonicWaveRear extends Weapon {

    private static final int FRAME = 9;
    private final float speed;

    public SonicWaveRear(RessourcesHandler rh) {
        super(rh, "Rear Sonic Wave", 16, 850);
        this.level.set(1);
        this.speed = 4.0f;
        this.front = false;
    }

    @Override
    protected void fire(Ship ship) {
        int dmg = 25;
        this.setRate(200);
        this.setEnergy(100);
        this.setCollision(0, 1, 0, 1);
        switch (this.level.getCurrent()) {
            case 1:
                this.leftRight(ship, dmg);
                break;
            case 2:
                this.addBullet(ship, this.l, dmg, FRAME, 0.0f, this.speed, 0, 0); // top
                this.leftRight(ship, dmg);
                break;
            case 3:
                this.dualFront(ship, dmg);
                this.leftRight(ship, dmg);
                break;
            case 4:
                this.dualFront2(ship, dmg);
                this.dualLeftRight(ship, dmg);
                break;
            case 5:
                this.dualFront2(ship, dmg);
                this.dualLeftRight(ship, dmg);
                this.dualDown(ship, dmg);
                break;
            case 6:
                this.dualFront2(ship, dmg);
                this.dualLeftRight(ship, dmg);
                this.dualDown(ship, dmg);
                this.addBullet(ship, this.l, dmg, FRAME + 16, 0.0f, -this.speed, 0, 0); // down
                this.addBullet(ship, this.l, dmg, FRAME + 90, -this.speed, -this.speed / 2, 0, 0); // down - left
                this.addBullet(ship, this.l, dmg, FRAME + 105, this.speed, -this.speed / 2, 0, 0); // down - right
                break;
        }
        SoundManager.play(SoundManager.SFX.WEAPON_WAVE);
    }

    private void dualFront(Ship ship, int dmg) {
        this.addBullet(ship, this.l, dmg, FRAME + 62, -this.speed / 4, this.speed, 0, 0); // top - left
        this.addBullet(ship, this.l, dmg, FRAME + 77, this.speed / 4, this.speed, 0, 0); // top - right
    }

    private void dualFront2(Ship ship, int dmg) {
        this.addBullet(ship, this.l, dmg, FRAME + 61, -this.speed / 1.5f, this.speed, 0, 0); // top - left2
        this.addBullet(ship, this.l, dmg, FRAME + 76, this.speed / 1.5f, this.speed, 0, 0); // top - right2
    }

    private void leftRight(Ship ship, int dmg) {
        this.addBullet(ship, this.l, dmg, FRAME + 30, -this.speed, 0.0f, 0, 0); // left
        this.addBullet(ship, this.l, dmg, FRAME + 45, this.speed, 0.0f, 0, 0); // right
    }

    private void dualLeftRight(Ship ship, int dmg) {
        this.addBullet(ship, this.l, dmg, FRAME + 60, -this.speed, this.speed / 4, 0, 0); // left - top
        this.addBullet(ship, this.l, dmg, FRAME + 90, -this.speed, -this.speed / 4, 0, 0); // left - down
        this.addBullet(ship, this.l, dmg, FRAME + 75, this.speed, this.speed / 4, 0, 0); // right - top
        this.addBullet(ship, this.l, dmg, FRAME + 105, this.speed, -this.speed / 4, 0, 0); // right - down
    }

    private void dualDown(Ship ship, int dmg) {
        this.addBullet(ship, this.l, dmg, FRAME + 91, -this.speed, -this.speed, 0, 0); // down - left2
        this.addBullet(ship, this.l, dmg, FRAME + 106, this.speed, -this.speed, 0, 0); // down - right2
        this.addBullet(ship, this.l, dmg, FRAME + 92, -this.speed / 2, -this.speed, 0, 0); // down - left3
        this.addBullet(ship, this.l, dmg, FRAME + 107, this.speed / 2, -this.speed, 0, 0); // down - right3
    }
}
